@@ -8,11 +8,11 @@ import itertools
 import json
 import random
 import re
-from typing import Any, Dict, List, Iterable, Optional, Union
+from typing import Any, Dict, List, Iterable, Optional
 
 import attr
-import dateutil
 
+from swh.core.api.serializers import msgpack_loads, msgpack_dumps
 from swh.model.model import (
     Revision,
     Release,
@@ -794,53 +794,51 @@ class CassandraStorage:
 
         return origin_url
 
-    def origin_visit_add(
-        self, origin_url: str, date: Union[str, datetime.datetime], type: str
-    ) -> OriginVisit:
-        if isinstance(date, str):
-            # FIXME: Converge on iso8601 at some point
-            date = dateutil.parser.parse(date)
-        elif not isinstance(date, datetime.datetime):
-            raise StorageArgumentException("Date must be a datetime or a string")
+    def origin_visit_add(self, visits: Iterable[OriginVisit]) -> Iterable[OriginVisit]:
+        for visit in visits:
+            origin = self.origin_get({"url": visit.origin})
+            if not origin:  # Cannot add a visit without an origin
+                raise StorageArgumentException("Unknown origin %s", visit.origin)
 
-        if not self.origin_get_one({"url": origin_url}):
-            raise StorageArgumentException("Unknown origin %s", origin_url)
+        all_visits = []
+        nb_visits = 0
+        for visit in visits:
+            nb_visits += 1
+            if not visit.visit:
+                visit_id = self._cql_runner.origin_generate_unique_visit_id(
+                    visit.origin
+                )
+                visit = attr.evolve(visit, visit=visit_id)
+            self.journal_writer.origin_visit_add([visit])
+            self._cql_runner.origin_visit_add_one(visit)
+            assert visit.visit is not None
+            all_visits.append(visit)
 
-        visit_id = self._cql_runner.origin_generate_unique_visit_id(origin_url)
-        visit_state = "ongoing"
-        with convert_validation_exceptions():
-            visit = OriginVisit.from_dict(
-                {
-                    "origin": origin_url,
-                    "date": date,
-                    "type": type,
-                    "status": visit_state,
-                    "snapshot": None,
-                    "metadata": None,
-                    "visit": visit_id,
-                }
-            )
+            visit_status_dict = visit.to_dict()
+            visit_status_dict.pop("type")
+            visit_status = OriginVisitStatus.from_dict(visit_status_dict)
+            self._origin_visit_status_add(visit_status)
 
-        self.journal_writer.origin_visit_add([visit])
-        self._cql_runner.origin_visit_add_one(visit)
-
-        with convert_validation_exceptions():
-            visit_status = OriginVisitStatus(
-                origin=origin_url,
-                visit=visit_id,
-                date=date,
-                status=visit_state,
-                snapshot=None,
-                metadata=None,
-            )
-        self._origin_visit_status_add(visit_status)
-
-        return visit
+        return all_visits
 
     def _origin_visit_status_add(self, visit_status: OriginVisitStatus) -> None:
         """Add an origin visit status"""
+        self.journal_writer.origin_visit_status_add([visit_status])
         self._cql_runner.origin_visit_status_add_one(visit_status)
 
+    def origin_visit_status_add(
+        self, visit_statuses: Iterable[OriginVisitStatus]
+    ) -> None:
+        # First round to check existence (fail early if any is ko)
+        for visit_status in visit_statuses:
+            origin_url = self.origin_get({"url": visit_status.origin})
+            if not origin_url:
+                raise StorageArgumentException(f"Unknown origin {visit_status.origin}")
+
+        self.journal_writer.origin_visit_status_add(visit_statuses)
+        for visit_status in visit_statuses:
+            self._origin_visit_status_add(visit_status)
+
     def origin_visit_update(
         self,
         origin: str,
@@ -1090,22 +1088,39 @@ class CassandraStorage:
         origin_url: str,
         authority: Dict[str, str],
         after: Optional[datetime.datetime] = None,
-        limit: Optional[int] = None,
-    ) -> List[Dict[str, Any]]:
+        page_token: Optional[bytes] = None,
+        limit: int = 1000,
+    ) -> Dict[str, Any]:
         if not isinstance(origin_url, str):
             raise TypeError("origin_url must be str, not %r" % (origin_url,))
 
-        if after is None:
+        if page_token is not None:
+            (after_date, after_fetcher_name, after_fetcher_url) = msgpack_loads(
+                page_token
+            )
+            if after and after_date < after:
+                raise StorageArgumentException(
+                    "page_token is inconsistent with the value of 'after'."
+                )
+            entries = self._cql_runner.origin_metadata_get_after_date_and_fetcher(
+                origin_url,
+                authority["type"],
+                authority["url"],
+                after_date,
+                after_fetcher_name,
+                after_fetcher_url,
+            )
+        elif after is not None:
+            entries = self._cql_runner.origin_metadata_get_after_date(
+                origin_url, authority["type"], authority["url"], after
+            )
+        else:
             entries = self._cql_runner.origin_metadata_get(
                 origin_url, authority["type"], authority["url"]
             )
-        else:
-            entries = self._cql_runner.origin_metadata_get_after(
-                origin_url, authority["type"], authority["url"], after
-            )
 
         if limit:
-            entries = itertools.islice(entries, 0, limit)
+            entries = itertools.islice(entries, 0, limit + 1)
 
         results = []
         for entry in entries:
@@ -1126,7 +1141,25 @@ class CassandraStorage:
                     "metadata": entry.metadata,
                 }
             )
-        return results
+
+        if len(results) > limit:
+            results.pop()
+            assert len(results) == limit
+            last_result = results[-1]
+            next_page_token: Optional[bytes] = msgpack_dumps(
+                (
+                    last_result["discovery_date"],
+                    last_result["fetcher"]["name"],
+                    last_result["fetcher"]["version"],
+                )
+            )
+        else:
+            next_page_token = None
+
+        return {
+            "next_page_token": next_page_token,
+            "results": results,
+        }
 
     def metadata_fetcher_add(
         self, name: str, version: str, metadata: Dict[str, Any]
