@@ -5,7 +5,7 @@
 
 import datetime
 
-from typing import Any, Dict, Iterable, List, Optional, Union
+from typing import Any, Dict, Iterable, List, Optional
 
 from swh.core.api import remote_api_endpoint
 from swh.model.model import (
@@ -13,6 +13,7 @@ from swh.model.model import (
     Directory,
     Origin,
     OriginVisit,
+    OriginVisitStatus,
     Revision,
     Release,
     Snapshot,
@@ -779,24 +780,32 @@ class StorageInterface:
         ...
 
     @remote_api_endpoint("origin/visit/add")
-    def origin_visit_add(
-        self, origin_url: str, date: Union[str, datetime.datetime], type: str
-    ) -> OriginVisit:
-        """Add an origin_visit for the origin at ts with status 'ongoing'.
+    def origin_visit_add(self, visits: Iterable[OriginVisit]) -> Iterable[OriginVisit]:
+        """Add visits to storage. If the visits have no id, they will be created and assigned
+        one. The resulted visits are visits with their visit id set.
 
         Args:
-            origin_url: visited origin identifier (its URL)
-            date: timestamp of such visit
-            type: the type of loader used for the visit (hg, git, ...)
+            visits: Iterable of OriginVisit objects to add
 
         Raises:
-            StorageArgumentException if the date is mistyped, or the origin
-            is unknown.
+            StorageArgumentException if some origin visit reference unknown origins
 
         Returns:
-            dict: dictionary with keys origin and visit where:
-            - origin: origin object
-            - visit: the visit object for the new visit occurrence
+            Iterable[OriginVisit] stored
+
+        """
+        ...
+
+    @remote_api_endpoint("origin/visit_status/add")
+    def origin_visit_status_add(
+        self, visit_statuses: Iterable[OriginVisitStatus],
+    ) -> None:
+        """Add origin visit statuses.
+
+        Args:
+            visit_statuses: origin visit statuses to add
+
+        Raises: StorageArgumentException if the origin of the visit status is unknown
 
         """
         ...
@@ -1147,6 +1156,9 @@ class StorageInterface:
         The authority and fetcher must be known to the storage before
         using this endpoint.
 
+        If there is already origin metadata for the same origin, authority,
+        fetcher, and at the same date, it will be replaced by this one.
+
         Args:
             discovery_date: when the metadata was fetched.
             authority: a dict containing keys `type` and `url`.
@@ -1162,18 +1174,23 @@ class StorageInterface:
         origin_url: str,
         authority: Dict[str, str],
         after: Optional[datetime.datetime] = None,
-        limit: Optional[int] = None,
-    ) -> List[Dict[str, Any]]:
+        page_token: Optional[bytes] = None,
+        limit: int = 1000,
+    ) -> Dict[str, Any]:
         """Retrieve list of all origin_metadata entries for the origin_id
 
         Args:
             origin_url: the origin's URL
             authority: a dict containing keys `type` and `url`.
             after: minimum discovery_date for a result to be returned
+            page_token: opaque token, used to get the next page of results
             limit: maximum number of results to be returned
 
         Returns:
-            list of dicts in the format:
+            dict with keys `next_page_token` and `results`.
+            `next_page_token` is an opaque token that is used to get the
+            next page of results, or `None` if there are no more results.
+            `results` is a list of dicts in the format:
 
             .. code-block: python
 
