@@ -9,14 +9,14 @@ import itertools
 
 from collections import defaultdict
 from contextlib import contextmanager
-from typing import Any, Dict, Iterable, List, Optional, Union
+from typing import Any, Dict, Iterable, List, Optional
 
 import attr
-import dateutil.parser
 import psycopg2
 import psycopg2.pool
 import psycopg2.errors
 
+from swh.core.api.serializers import msgpack_loads, msgpack_dumps
 from swh.model.model import (
     Content,
     Directory,
@@ -823,68 +823,61 @@ class Storage:
     @timed
     @db_transaction()
     def origin_visit_add(
-        self,
-        origin_url: str,
-        date: Union[str, datetime.datetime],
-        type: str,
-        db=None,
-        cur=None,
-    ) -> OriginVisit:
-        if isinstance(date, str):
-            # FIXME: Converge on iso8601 at some point
-            date = dateutil.parser.parse(date)
-        elif not isinstance(date, datetime.datetime):
-            raise StorageArgumentException("Date must be a datetime or a string")
+        self, visits: Iterable[OriginVisit], db=None, cur=None
+    ) -> Iterable[OriginVisit]:
+        for visit in visits:
+            origin = self.origin_get({"url": visit.origin}, db=db, cur=cur)
+            if not origin:  # Cannot add a visit without an origin
+                raise StorageArgumentException("Unknown origin %s", visit.origin)
 
-        origin = self.origin_get({"url": origin_url}, db=db, cur=cur)
-        if not origin:  # Cannot add a visit without an origin
-            raise StorageArgumentException("Unknown origin %s", origin_url)
+        all_visits = []
+        nb_visits = 0
+        for visit in visits:
+            nb_visits += 1
+            if not visit.visit:
+                with convert_validation_exceptions():
+                    visit_id = db.origin_visit_add(
+                        visit.origin, visit.date, visit.type, cur=cur
+                    )
+                visit = attr.evolve(visit, visit=visit_id)
+            else:
+                db.origin_visit_upsert(visit)
+            assert visit.visit is not None
+            all_visits.append(visit)
+            # Forced to write after for the case when the visit has no id
+            self.journal_writer.origin_visit_add([visit])
+            visit_status_dict = visit.to_dict()
+            visit_status_dict.pop("type")
+            visit_status = OriginVisitStatus.from_dict(visit_status_dict)
+            self._origin_visit_status_add(visit_status, db=db, cur=cur)
 
-        with convert_validation_exceptions():
-            visit_id = db.origin_visit_add(origin_url, date, type, cur=cur)
-
-        status = "ongoing"
-        # We can write to the journal only after inserting to the
-        # DB, because we want the id of the visit
-        visit = OriginVisit.from_dict(
-            {
-                "origin": origin_url,
-                "date": date,
-                "type": type,
-                "visit": visit_id,
-                # TODO: Remove when we remove those fields from the model
-                "status": status,
-                "metadata": None,
-                "snapshot": None,
-            }
-        )
-
-        with convert_validation_exceptions():
-            visit_status = OriginVisitStatus(
-                origin=origin_url,
-                visit=visit_id,
-                date=date,
-                status=status,
-                snapshot=None,
-                metadata=None,
-            )
-        self._origin_visit_status_add(visit_status, db=db, cur=cur)
-
-        self.journal_writer.origin_visit_add([visit])
-
-        send_metric("origin_visit:add", count=1, method_name="origin_visit")
-        return visit
+        send_metric("origin_visit:add", count=nb_visits, method_name="origin_visit")
+        return all_visits
 
     def _origin_visit_status_add(
-        self, origin_visit_status: OriginVisitStatus, db, cur
+        self, visit_status: OriginVisitStatus, db, cur
     ) -> None:
         """Add an origin visit status"""
-        db.origin_visit_status_add(origin_visit_status, cur=cur)
-        # TODO: write to the journal the origin visit status
+        self.journal_writer.origin_visit_status_add([visit_status])
+        db.origin_visit_status_add(visit_status, cur=cur)
         send_metric(
             "origin_visit_status:add", count=1, method_name="origin_visit_status"
         )
 
+    @timed
+    @db_transaction()
+    def origin_visit_status_add(
+        self, visit_statuses: Iterable[OriginVisitStatus], db=None, cur=None,
+    ) -> None:
+        # First round to check existence (fail early if any is ko)
+        for visit_status in visit_statuses:
+            origin_url = self.origin_get({"url": visit_status.origin}, db=db, cur=cur)
+            if not origin_url:
+                raise StorageArgumentException(f"Unknown origin {visit_status.origin}")
+
+        for visit_status in visit_statuses:
+            self._origin_visit_status_add(visit_status, db, cur)
+
     @timed
     @db_transaction()
     def origin_visit_update(
@@ -941,7 +934,7 @@ class Storage:
                     snapshot=snapshot or last_visit_status["snapshot"],
                     metadata=metadata or last_visit_status["metadata"],
                 )
-            self._origin_visit_status_add(visit_status, db=db, cur=cur)
+                self._origin_visit_status_add(visit_status, db=db, cur=cur)
 
     def _origin_visit_get_updated(
         self, origin: str, visit_id: int, db, cur
@@ -993,26 +986,35 @@ class Storage:
     def origin_visit_upsert(
         self, visits: Iterable[OriginVisit], db=None, cur=None
     ) -> None:
+        visit_statuses = []
+        nb_visits = 0
         for visit in visits:
+            nb_visits += 1
             if visit.visit is None:
                 raise StorageArgumentException(f"Missing visit id for visit {visit}")
+            with convert_validation_exceptions():
+                visit_statuses.append(
+                    OriginVisitStatus(
+                        origin=visit.origin,
+                        visit=visit.visit,
+                        date=now(),
+                        status=visit.status,
+                        snapshot=visit.snapshot,
+                        metadata=visit.metadata,
+                    )
+                )
 
+        assert len(visit_statuses) == nb_visits
+
+        # write in journal first
         self.journal_writer.origin_visit_upsert(visits)
+        self.journal_writer.origin_visit_status_add(visit_statuses)
 
-        for visit in visits:
-            # TODO: upsert them all in a single query
+        # then sync to db
+        for i, visit in enumerate(visits):
             assert visit.visit is not None
             db.origin_visit_upsert(visit, cur=cur)
-            with convert_validation_exceptions():
-                visit_status = OriginVisitStatus(
-                    origin=visit.origin,
-                    visit=visit.visit,
-                    date=now(),
-                    status=visit.status,
-                    snapshot=visit.snapshot,
-                    metadata=visit.metadata,
-                )
-            db.origin_visit_status_add(visit_status, cur=cur)
+            db.origin_visit_status_add(visit_statuses[i], cur=cur)
 
     @timed
     @db_transaction_generator(statement_timeout=500)
@@ -1278,18 +1280,38 @@ class Storage:
         origin_url: str,
         authority: Dict[str, str],
         after: Optional[datetime.datetime] = None,
-        limit: Optional[int] = None,
+        page_token: Optional[bytes] = None,
+        limit: int = 1000,
         db=None,
         cur=None,
-    ) -> List[Dict[str, Any]]:
+    ) -> Dict[str, Any]:
+        if page_token:
+            (after_time, after_fetcher) = msgpack_loads(page_token)
+            if after and after_time < after:
+                raise StorageArgumentException(
+                    "page_token is inconsistent with the value of 'after'."
+                )
+        else:
+            after_time = after
+            after_fetcher = None
+
         authority_id = db.metadata_authority_get_id(
             authority["type"], authority["url"], cur
         )
         if not authority_id:
-            return []
+            return {
+                "next_page_token": None,
+                "results": [],
+            }
+
+        rows = db.origin_metadata_get(
+            origin_url, authority_id, after_time, after_fetcher, limit + 1, cur
+        )
+        rows = [dict(zip(db.origin_metadata_get_cols, row)) for row in rows]
         results = []
-        for line in db.origin_metadata_get(origin_url, authority_id, after, limit, cur):
-            row = dict(zip(db.origin_metadata_get_cols, line))
+        for row in rows:
+            row = row.copy()
+            row.pop("metadata_fetcher.id")
             results.append(
                 {
                     "origin_url": row.pop("origin.url"),
@@ -1304,7 +1326,24 @@ class Storage:
                     **row,
                 }
             )
-        return results
+
+        if len(results) > limit:
+            results.pop()
+            assert len(results) == limit
+            last_returned_row = rows[-2]  # rows[-1] corresponds to the popped result
+            next_page_token: Optional[bytes] = msgpack_dumps(
+                (
+                    last_returned_row["discovery_date"],
+                    last_returned_row["metadata_fetcher.id"],
+                )
+            )
+        else:
+            next_page_token = None
+
+        return {
+            "next_page_token": next_page_token,
+            "results": results,
+        }
 
     @timed
     @db_transaction()
