@@ -18,10 +18,30 @@ from typing import Dict
 import swh.storage
 
 from swh.core.utils import numfile_sortkey as sortkey
-
 from swh.model.tests.generate_testdata import gen_contents, gen_origins
+from swh.model.model import (
+    Content,
+    Directory,
+    Origin,
+    OriginVisit,
+    Release,
+    Revision,
+    SkippedContent,
+    Snapshot,
+)
 
 
+OBJECT_FACTORY = {
+    "content": Content.from_dict,
+    "directory": Directory.from_dict,
+    "origin": Origin.from_dict,
+    "origin_visit": OriginVisit.from_dict,
+    "release": Release.from_dict,
+    "revision": Revision.from_dict,
+    "skipped_content": SkippedContent.from_dict,
+    "snapshot": Snapshot.from_dict,
+}
+
 SQL_DIR = path.join(path.dirname(swh.storage.__file__), "sql")
 
 environ["LC_ALL"] = "C.UTF-8"
@@ -51,10 +71,7 @@ def swh_storage_backend_config(postgresql_proc, swh_storage_postgresql):
 
 @pytest.fixture
 def swh_storage(swh_storage_backend_config):
-    storage_config = {"cls": "validate", "storage": swh_storage_backend_config}
-
-    storage = swh.storage.get_storage(**storage_config)
-    return storage
+    return swh.storage.get_storage(cls="validate", storage=swh_storage_backend_config)
 
 
 @pytest.fixture
@@ -229,7 +246,7 @@ def sample_data() -> Dict:
         "release": [data.release, data.release2, data.release3],
         "snapshot": [data.snapshot],
         "origin": [data.origin, data.origin2],
-        "tool": [data.metadata_tool],
-        "provider": [data.provider],
+        "fetcher": [data.metadata_fetcher],
+        "authority": [data.metadata_authority],
         "origin_metadata": [data.origin_metadata, data.origin_metadata2],
     }
